#import "MessageClient.h"
#import "bsd/m68k/machparam.h"
#import "bsd/sys/fcntl.h"

#define NUMMESSAGES 100

void main()
{
	kern_return_t error;
	port_t serverPort, localPort;
	msg_return_t returnCode;
	char hostname[50];
	int count, fileDescriptor;
	char data[100];
	port_t notifyPort;

	// Set up a notification port.
	if ((error=port_allocate(task_self(), &notifyPort)) != KERN_SUCCESS) 
	{
    	mach_error("notify port_allocate failed", error); 
    	exit(1);
	}
	
	// Really make it a notification port.
	task_set_special_port(task_self(), TASK_NOTIFY_PORT, notifyPort);

	// Open the file with read and write permissions for the owner only.
	fileDescriptor = open("/tmp/distribute", O_WRONLY | O_CREAT | O_TRUNC,
			384);
	
	// Get the server's port.
	if( (error = netname_look_up(name_server_port, "reis27", "MESSAGESERVER",
			&serverPort)) != NETNAME_SUCCESS )
		exit(1);

	// Set up a local port.
	if ((error=port_allocate(task_self(), &localPort)) != KERN_SUCCESS) 
	{
    	mach_error("port_allocate failed", error); 
    	exit(1);
	}
	
	// Set up the messages to be sent to the server.
	messageFromClient.msgHeader.msg_simple = TRUE;
	messageFromClient.msgHeader.msg_size = sizeof(struct messageStruct);
	messageFromClient.msgHeader.msg_type = MSG_TYPE_NORMAL; 
	messageFromClient.msgHeader.msg_local_port = localPort;
	messageFromClient.msgHeader.msg_remote_port = serverPort; 
	messageFromClient.msgHeader.msg_id = 0; 
	
	messageFromClient.msgType.msg_type_name = MSG_TYPE_INTEGER_32;
	messageFromClient.msgType.msg_type_size = 32;
	messageFromClient.msgType.msg_type_number = 2;
	messageFromClient.msgType.msg_type_inline = TRUE;
	messageFromClient.msgType.msg_type_longform = FALSE; 
	messageFromClient.msgType.msg_type_deallocate = FALSE;

	// Set up the notify header.
	notifyMessage.notify_header.msg_local_port = notifyPort;
	notifyMessage.notify_header.msg_size = sizeof(notification_t);

	gethostname(hostname, 50);
	messageFromClient.machineNumber = atoi(hostname+4);
	
	// Send messages to the server.
	for(count = 0; count < NUMMESSAGES; count++)
	{
		messageFromClient.data = count;
 		returnCode = msg_send(&messageFromClient.msgHeader, 
 			SEND_TIMEOUT | SEND_NOTIFY, 1000);
		
		if(returnCode == SEND_WILL_NOTIFY)
		{
			sprintf(data, "sendWillNotify received, message num %d \n", count);
			write(fileDescriptor, data, strlen(data));

 			returnCode = msg_receive(&notifyMessage.notify_header, 
					MSG_OPTION_NONE, 0);
			if(returnCode ==RCV_SUCCESS)
			{
				if(notifyMessage.notify_header.msg_id == NOTIFY_MSG_ACCEPTED) 
					sprintf(data, "notifyMsgAccepted received\n");
				else if(notifyMessage.notify_header.msg_id == 
						NOTIFY_PORT_DELETED) 
        			sprintf(data, "notifyPortDeleted\n");
				else
				{ 
					if(notifyMessage.notify_port == serverPort)
	     				sprintf(data, "correct port, unknown error\n");
					else
						sprintf(data, "wrong port, unknown error\n");
				}
				write(fileDescriptor, data, strlen(data));
			}
			else
			{
				sprintf(data, "error in msg_Receive\n");
				write(fileDescriptor, data, strlen(data));
			}
		}
		else if (returnCode != SEND_SUCCESS)
		{
			sprintf(data, "bad return code \n");
			write(fileDescriptor, data, strlen(data));
		}		
	}		
	return;
}	
